@@ -10,15 +10,19 @@
 #   anyone's safety, is implied in this software or documentation.
 #
 #   This specific module is licensed under the Perl Artistic License.
+#   Effective 28 January 2021, the original author and copyright holder, 
+#   Martin Hosken, has given permission to use and redistribute this module 
+#   under the MIT license.
 #
 #=======================================================================
 package PDF::Builder::Basic::PDF::File;
 
 use strict;
-no warnings qw[ deprecated recursion uninitialized ];
+use warnings;
+#no warnings qw[ deprecated recursion uninitialized ];
 
-our $VERSION = '3.019'; # VERSION
-my $LAST_UPDATE = '3.018'; # manually update whenever code is changed
+our $VERSION = '3.022'; # VERSION
+my $LAST_UPDATE = '3.022'; # manually update whenever code is changed
 
 =head1 NAME
 
@@ -419,7 +423,7 @@ sub out_file {
 =head2 $p->create_file($fname)
 
 Creates a new output file (no check is made of an existing open file) of
-the given filename or IO object. Note, make sure that C<< $p->{' version'} >> 
+the given filename or IO object. Note: make sure that C<< $p->{' version'} >>
 is set correctly before calling this function.
 
 =cut
@@ -469,7 +473,7 @@ sub close_file {
     $tdict->{'Size'} = $self->{'Size'} || PDFNum(1);
     $tdict->{'Prev'} = PDFNum($self->{' loc'}) if $self->{' loc'};
     if ($self->{' update'}) {
-        foreach my $key (grep ($_ !~ m/^[\s\-]/, keys %$self)) { ## no critic
+        foreach my $key (grep { $_ !~ m/^[\s\-]/ } keys %$self) {
             $tdict->{$key} = $self->{$key} unless defined $tdict->{$key};
         }
 
@@ -936,8 +940,8 @@ sub remove_obj {
     delete $self->{' objects'}{$objind->uid()};
     delete $self->{' outlist_cache'}{$objind};
     delete $self->{' printed_cache'}{$objind};
-    @{$self->{' outlist'}} = grep($_ ne $objind, @{$self->{' outlist'}}); ## no critic
-    @{$self->{' printed'}} = grep($_ ne $objind, @{$self->{' printed'}}); ## no critic
+    @{$self->{' outlist'}} = grep { $_ ne $objind } @{$self->{' outlist'}};
+    @{$self->{' printed'}} = grep { $_ ne $objind } @{$self->{' printed'}};
     $self->{' objcache'}{$objind->{' objnum'}, $objind->{' objgen'}} = undef
         if $self->{' objcache'}{$objind->{' objnum'}, $objind->{' objgen'}} eq $objind;
 
@@ -984,7 +988,7 @@ sub ship_out {
         my ($objnum, $objgen) = @{$self->{' objects'}{$objind->uid()}}[0..1];
         $fh->printf('%d %d obj ', $objnum, $objgen);
         $objind->outobjdeep($fh, $self);
-        $fh->print(" endobj\n");
+        $fh->print("\nendobj\n");
 
         # Note that we've output this obj, not forgetting to update
         # the cache of what's printed.
@@ -1168,13 +1172,45 @@ See C<open> for options allowed.
 sub _unpack_xref_stream {
     my ($self, $width, $data) = @_;
 
-    return unpack('C', $data)       if $width == 1;
-    return unpack('n', $data)       if $width == 2;
-    return unpack('N', "\x00$data") if $width == 3;
-    return unpack('N', $data)       if $width == 4;
-    return unpack('Q', $data)       if $width == 8; # PDF 1.5+?
+    # handle some oddball cases
+    if      ($width == 3) {
+	$data = "\x00$data";
+	$width = 4;
+    } elsif ($width == 5) {
+	$data = "\x00\x00\x00$data";
+	$width = 8;
+    } elsif ($width == 6) {
+	$data = "\x00\x00$data";
+	$width = 8;
+    } elsif ($width == 7) {
+	$data = "\x00$data";
+	$width = 8;
+    }
+    # in all cases, "Network" (Big-Endian) byte order assumed
+    return unpack('C', $data)  if $width == 1;
+    return unpack('n', $data)  if $width == 2;
+    return unpack('N', $data)  if $width == 4;
+    if ($width == 8) {
+	# Some ways other packages handle this, without Perl-64, according
+	# to Vadim Repin. Possibly they end up converting the value to
+	# "double" behind the scenes if on a 32-bit platform?
+	# PDF::Tiny  return hex unpack('H16', $data);
+	# CAM::PDF   my @b = unpack('C*', $data); 
+	#            my $i=0; ($i <<= 8) += shift @b while @b; return $i;
+	
+	if (substr($data, 0, 4) eq "\x00\x00\x00\x00") {
+	    # can treat as 32 bit unsigned int
+            return unpack('N', substr($data, 4, 4));
+	} else {
+            # requires 64-bit platform (chip and Perl), else fatal error
+	    # it may blow up and produce a smoking crater if 32-bit Perl!
+            # also note that Q needs Big-Endian flag (>) specified, else 
+	    #   it will use the native chip order (Big- or Little- Endian)
+            return unpack('Q>', $data);
+	}
+    }
 
-    die "Invalid column width: $width";
+    die "Unsupported field width: $width. 1-8 supported.";
 }
 
 sub readxrtr {
@@ -1236,8 +1272,9 @@ sub readxrtr {
             my $entry_size = 20;
             # test read first entry, see if $cr in expected place, adjust size
             $fh->read($buf, $entry_size * 1 - $xdiff + 15, $xdiff);
-            $buf =~ m/^(.*?)$cr/;
-            $entry_size = length($1) + 2;
+            if ($buf =~ m/^(.*?)$cr/) {
+                $entry_size = length($1) + 2;
+	    }
             if ($entry_size != 20 && $options{'-diags'} == 1) {
                 warn "Xref entries supposed to be 20 bytes long, are $entry_size.\n";
             }
@@ -1385,6 +1422,7 @@ sub readxrtr {
     } elsif ($buf =~ m/^(\d+)\s+(\d+)\s+obj/i) {
         my ($xref_obj, $xref_gen) = ($1, $2);
 
+	PDF::Builder->verCheckOutput(1.5, "importing cross-reference stream");
         # XRef streams
         ($tdict, $buf) = $self->readval($buf);
 
